#    Inside The Data Tunnel - My game for PyWeek 20.
#    Copyright (C) 2015  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Modules
# Standard modules
import random
import copy
import sqlite3
import os
import os.path

# Third party modules
import pygame

# Own game modules
from config import config
from classes.Passive_Data_Sprite import Passive_Data_Sprite
from modules import helpers
from config import Virus_Config
from config import Worm_Config
from config import Trojan_Horse_Config
from config import Backdoor_Config
from config import Rootkit_Config
from classes.Malware import Malware
from classes.Player1 import Player1
from classes.Player2 import Player2
from config import Player1_Robot_Config
from config import Player2_Robot_Config
from config import Ally_Robot_Config
from classes.Ally import Ally
from config import levels
from classes.Menu import Menu
from classes.Text import Text


# Initialise pygame, so that we can use its' modules.
pygame.init()

pygame.mixer.music.load(
	os.path.join(
		config.MUSIC_PATH,
		config.MUSIC_FILENAME
	)
)
pygame.mixer.music.set_volume( 0.1 )
pygame.mixer.music.play( -1 )

def main():
	"""This is the main game loop.

	"""
	# We need a window so that we can display the graphics.  This is what the
	# user will see.  Think of it as a window in Windows or Linux.
	# The window will be created with extra modes if at least one mode has been
	# set "config.WINDOW_MODES".  Take a look at "config/config.py" if you want
	# to know more.
	if config.WINDOW_MODES is 0:
		window = pygame.display.set_mode(
			config.WINDOW_DIMENSIONS
		)
	else:
		window = pygame.display.set_mode(
			config.WINDOW_DIMENSIONS,
			config.WINDOW_MODES
		)

	# Do not show the mouse pointer.
	pygame.mouse.set_visible( False )

	
	# We need a clock object for controlling how often we want pygame to refresh
	# the window.  This will also affect the speed of all of our game objects.
	clock = pygame.time.Clock()	

	###### testing only #####
	#show_enter_name_for_highscore_table_screen( window, clock, "multi" )
	#return

	# We have to set up a database for the highscores if it does not yet exist.
	# I am going to use SQLite3 for this task.
	db_conn = sqlite3.connect( config.DB_NAME )
	db_cursor = db_conn.cursor()
	db_cursor.execute( """CREATE TABLE IF NOT EXISTS highscore_single (
		name TEXT, score INTEGER )""" )
	db_conn.commit()
	db_cursor.execute( """CREATE TABLE IF NOT EXISTS highscore_multi (
		name TEXT, score INTEGER )""" )
	db_conn.commit()
	db_conn.close()
 
	
	# Run the game!
	run_main_menu( window, clock )

	# The user wants to quit the game.  The last remaining thing we need to do
	# is kill the window, or otherwise the user will get a warning in the
	# console window.
	pygame.display.quit()


def run_main_menu( window, clock ):
	"""

	"""

	# The main menu
	main_menu = Menu( config.MAIN_MENU )
	choose_players_menu = Menu( config.CHOOSE_PLAYERS_MENU )
	highscores_menu = Menu( config.HIGHSCORES_MENU )
	help_menu = Menu( config.HELP_MENU )
	story_menu = Menu( config.STORY_MENU )
	singleplayer_highscores_menu = Menu( config.SINGLEPLAYER_HIGHSCORE_MENU )
	multiplayer_highscores_menu = Menu( config.MULTIPLAYER_HIGHSCORE_MENU )	
		
	running = True
	while running:
		eventlist = pygame.event.get()

		for event in eventlist:
			if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
				running = False
				
		window.fill( config.GREY1 )

		main_menu.update( eventlist )
		main_menu.draw( window )

		if main_menu.action == "pick_players":
			run_choose_players_menu = True
			
			while run_choose_players_menu:
			
				eventlist = pygame.event.get()
				
				window.fill( config.GREY1 )
				choose_players_menu.update( eventlist )
					
				main_menu.draw( window )
				choose_players_menu.draw( window )

				if choose_players_menu.action == "return":
					run_choose_players_menu = False
				elif choose_players_menu.action == "run_singleplayer_game":
					run_singleplayer_game( window, clock )
					run_choose_players_menu = False
				elif choose_players_menu.action == "run_multiplayer_game":
					run_multiplayer_game( window, clock )
					run_choose_players_menu = False
				
				pygame.display.update()
				clock.tick( config.FPS )
						
			choose_players_menu.reset()

		elif main_menu.action == "highscores":
			run_highscores_menu = True
			
			while run_highscores_menu:
			
				eventlist = pygame.event.get()
				
				window.fill( config.GREY1 )
				highscores_menu.update( eventlist )
					
				main_menu.draw( window )
				highscores_menu.draw( window )

				if highscores_menu.action == "return":
					run_highscores_menu = False
					
				elif highscores_menu.action == "singleplayer_highscores":
					run_singleplayer_highscores_menu = True

					# Load highscores for single player and display them.
					highscores_single = helpers.get_highscores( "single" )
					cur_pos = [ 300, 250 ]
					highscores_single_text = pygame.sprite.Group()
					counter1 = 1
					
					for h in highscores_single:
						highscores_single_text.add(
							Text(
								"1. %s - %d points" % ( h[0], h[1] ),
								14,
								config.WHITE,
								cur_pos
							)
						)

						cur_pos[1] += 15
						
					while run_singleplayer_highscores_menu:
			
						eventlist = pygame.event.get()
				
						window.fill( config.GREY1 )
						singleplayer_highscores_menu.update( eventlist )
					
						main_menu.draw( window )
						highscores_menu.draw( window )
						singleplayer_highscores_menu.draw( window )
						highscores_single_text.draw( window )

						if singleplayer_highscores_menu.action == "return":
							run_singleplayer_highscores_menu = False
					
						pygame.display.update()
						clock.tick( config.FPS )
						
					singleplayer_highscores_menu.reset()
					highscores_menu.action = ""
									
				elif highscores_menu.action == "multiplayer_highscores":
					run_multiplayer_highscores_menu = True

					# Load highscores for multiplayer and display them.					
					highscores_multi = helpers.get_highscores( "multi" )
					cur_pos = [ 300, 250 ]
					highscores_multi_text = pygame.sprite.Group()
					counter1 = 1
					
					for h in highscores_multi:
						highscores_multi_text.add(
							Text(
								"1. %s - %d points" % ( h[0], h[1] ),
								14,
								config.WHITE,
								cur_pos
							)
						)

						cur_pos[1] += 15
									
					while run_multiplayer_highscores_menu:
			
						eventlist = pygame.event.get()
				
						window.fill( config.GREY1 )
						multiplayer_highscores_menu.update( eventlist )
					
						main_menu.draw( window )
						highscores_menu.draw( window )
						multiplayer_highscores_menu.draw( window )
						highscores_multi_text.draw( window )

						if multiplayer_highscores_menu.action == "return":
							run_multiplayer_highscores_menu = False
					
						pygame.display.update()
						clock.tick( config.FPS )
						
					multiplayer_highscores_menu.reset()
					highscores_menu.action = ""					
				
				pygame.display.update()
				clock.tick( config.FPS )
						
			highscores_menu.reset()

		elif main_menu.action == "story":
			run_story_menu = True

			# Set up the text to display.
			story_menu_texts = config.STORY_MENU_TEXT.splitlines()
			cur_pos = [ 300, 250 ]
			story_menu_texts_group = pygame.sprite.Group()
			for x in story_menu_texts:
				story_menu_texts_group.add(
					Text(
						x,
						14,
						config.WHITE,
						cur_pos
					)
				)
				cur_pos[1] += 15
			
			while run_story_menu:
			
				eventlist = pygame.event.get()
				
				window.fill( config.GREY1 )
				story_menu.update( eventlist )
					
				main_menu.draw( window )
				story_menu.draw( window )
				story_menu_texts_group.draw( window )		

				if story_menu.action == "return":
					run_story_menu = False
				
				pygame.display.update()
				clock.tick( config.FPS )
						
			story_menu.reset()
			
		elif main_menu.action == "help":
			run_help_menu = True

			help_menu_player1_controls_text = Text( **config.HELP_MENU_PLAYER1_CONTROLS )
			help_menu_player2_controls_text = Text( **config.HELP_MENU_PLAYER2_CONTROLS )
			
			while run_help_menu:
			
				eventlist = pygame.event.get()
				
				window.fill( config.GREY1 )
				help_menu.update( eventlist )
					
				main_menu.draw( window )
				help_menu.draw( window )

				window.blit( help_menu_player1_controls_text.image, help_menu_player1_controls_text.rect )
				window.blit( help_menu_player2_controls_text.image, help_menu_player2_controls_text.rect )				

				if help_menu.action == "return":
					run_help_menu = False
				
				pygame.display.update()
				clock.tick( config.FPS )
						
			help_menu.reset()
			
		
			
		elif main_menu.action == "quit":
			running = False

		main_menu.action = ""
		
		pygame.display.update()
		clock.tick( config.FPS )

	pygame.display.quit()


def run_singleplayer_game( window, clock ):
	"""Runs the main game.

	"""
	# GOOD FORCE:
	allies_players = pygame.sprite.Group()
	players = pygame.sprite.Group()
	allies = pygame.sprite.Group()
		
	# Create the players first.
	player1 = Player1( **Player1_Robot_Config.CONFIG )

	allies_players.add( player1 )

	players.add( player1 )

	# Now create the allies!
	for ally in range( 10 ):
		ally = Ally( **Ally_Robot_Config.CONFIG )
		allies_players.add( ally )
		allies.add( ally )

	# Create the score indicator / counter.
	current_score = 0
	score_counter_config = copy.deepcopy( config.SCORE_INDICATOR )
	score_counter_config["_text"] = score_counter_config["_text"] % current_score
	score_counter = Text( **score_counter_config )

	# We need a sprite group for our items.
	items_group = pygame.sprite.Group()

	# We start with level 1.
	current_level_cnt = 1
	max_levels = len( levels.LEVELS )

	# Always show the current level in the top-left corner below the score counter.
	current_level_text_config = copy.deepcopy( config.CURRENT_LEVEL_TEXT )
	current_level_text_config["_text"] %= current_level_cnt
	current_level_text = Text( **current_level_text_config )

	game_aborted = False
	
	while not player1.is_dead:
		# Load and run the next level.
		current_level = levels.LEVELS[current_level_cnt]

		# Create malware first!!!
		# Malware sprite group.
		malware = pygame.sprite.Group()
		
		# Create viruses for the current level.
		for virus in range( current_level["viruses"] ):
			malware.add( Malware( **Virus_Config.LEVELS[current_level_cnt] ) )
			
		# Create worms for the current level.
		for worm in range( current_level["worms"] ):
			malware.add( Malware( **Worm_Config.LEVELS[current_level_cnt] ) )

		# Create trojan horses for the current level.
		for worm in range( current_level["trojan_horses"] ):
			malware.add( Malware( **Trojan_Horse_Config.LEVELS[current_level_cnt] ) )

		# Create backdoors for the current level.
		for worm in range( current_level["backdoors"] ):
			malware.add( Malware( **Backdoor_Config.LEVELS[current_level_cnt] ) )

		# Create rootkits for the current level.
		for worm in range( current_level["rootkits"] ):
			malware.add( Malware( **Rootkit_Config.LEVELS[current_level_cnt] ) )			
		
		# Create passive data that move from the left to the right.
		passive_data = pygame.sprite.Group()
		for x in range( 0, 10 ):
			passive_data.add(
				Passive_Data_Sprite(
					"Neutral_Data" + str(random.randint(1,3)) + ".png",
					90,
					random.randint( 2, 6 ),
					3.0,
					pygame.Rect( -200, 59, 1324, 650 )
				)
			)

		# We inform the user about the next level.
		show_level_screen( window, clock, current_level_cnt )

		# The main gameloop runs the game and keeps it running until the user
		# chooses to quit the game.
		# We are going to use a variable to control this behaviour.		
		run_level = True
		while run_level:
			# We check for keypresses that come from the keyboard of the user.
			# If the user presses the ESC-key, we quit the game-loop, and thus, the
			# game.
			# A keypress is an event.  All events are stored in a list, which I call
			# the eventlist.
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
					if show_pause_screen( window, clock ):
						run_level = False
						player1.is_dead = True
						game_aborted = True
						continue

			if player1.is_dead and not run_level:
				continue

			# We can draw on this window and fill it with colours.  The first thing
			# we need to do is fill the entire window with GREY1.  This is something
			# I like to do conveniently to keep the window refreshed.
			window.fill( config.GREY1 )

			passive_data.update( items_group )
			items_group.update( players, allies_players, malware )
			malware.update( allies_players )
			allies.update( malware )
			players.update( eventlist, malware )

			passive_data.draw( window )
			items_group.draw( window )
			
			for m in malware:
				m.draw( window )
			for a in allies:
				a.draw( window )
			for p in players:
				p.draw( window )

			if len( malware ) == 0:
				run_level = False
			if len( players ) == 0:
				run_level = False
				show_you_lose_screen( window, clock )

			# Update the score displayed.
			current_score = helpers.sum_score( player1.get_score() )
			score_counter.change_text( config.SCORE_INDICATOR["_text"] % current_score )

			# Blit the score indicator / counter
			window.blit( score_counter.image, score_counter.rect )
			window.blit( current_level_text.image, current_level_text.rect )

			# Now that we have made changes to the window we need to show them to
			# the user who plays our game.
			pygame.display.update()

			for m in malware:
				if m.has_entered_cpu:
					run_level = False
					player1.is_dead = True
					show_you_lose_screen( window, clock )	
					break			

			# We need to limit the game to a certain amount of updates per second.
			# If we would not do this, the computer would try to refresh or run
			# through the while-loop as quickly as possible.  This would result
			# in unstable playing, and also it would use so many CPU resources that
			# its' cooler would turn on, which can be really annoying.
			# We will run this game at a specified frame rate.
			clock.tick( config.FPS )


		current_level_cnt += 1
		if player1.is_dead:
			# Check if the player still made it into the highscore.
			if helpers.is_in_highscore( current_score, "single" ) and not game_aborted:
				# Show the screen where the player can enter his name, because
				# he made it into the highscore.
				player_name = show_enter_name_for_highscore_table_screen( window, clock, "single" )
				helpers.insert_new_highscore( player_name, current_score, "single" )				
			continue
			
		if current_level_cnt > max_levels:
			show_mission_accomplished_screen( window, clock, current_score )
			# Check if the player still made it into the highscore.
			if helpers.is_in_highscore( current_score, "single" ) and not game_aborted:
				# Show the screen where the player can enter his name, because
				# he made it into the highscore.
				player_name = show_enter_name_for_highscore_table_screen( window, clock, "single" )
				helpers.insert_new_highscore( player_name, current_score, "single" )			
			break
		else:
			# Reset the position of the allies.
			# And remove remaining projectiles.
			for a in allies:
				a.reset_position()
				a.projectiles.empty()

			# Always show the current level in the top-left corner below the score counter.
			current_level_text_config = copy.deepcopy( config.CURRENT_LEVEL_TEXT )
			current_level_text_config["_text"] %= current_level_cnt
			current_level_text = Text( **current_level_text_config )


def run_multiplayer_game( window, clock ):
	"""Runs the main game.

	"""
	# GOOD FORCE:
	allies_players = pygame.sprite.Group()
	players = pygame.sprite.Group()
	allies = pygame.sprite.Group()
		
	# Create the players first.
	player1 = Player1( **Player1_Robot_Config.CONFIG )
	player2 = Player2( **Player2_Robot_Config.CONFIG )

	allies_players.add( player1 )
	allies_players.add( player2 )

	players.add( player1 )
	players.add( player2 )

	# Now create the allies!
	for ally in range( 10 ):
		ally = Ally( **Ally_Robot_Config.CONFIG )
		allies_players.add( ally )
		allies.add( ally )

	# Create the score indicator / counter.
	current_score = 0
	score_counter_config = copy.deepcopy( config.SCORE_INDICATOR )
	score_counter_config["_text"] = score_counter_config["_text"] % current_score
	score_counter = Text( **score_counter_config )

	# We need a sprite group for our items.
	items_group = pygame.sprite.Group()	

	# We start with level 1.
	current_level_cnt = 1
	max_levels = len( levels.LEVELS )

	# Always show the current level in the top-left corner below the score counter.
	current_level_text_config = copy.deepcopy( config.CURRENT_LEVEL_TEXT )
	current_level_text_config["_text"] %= current_level_cnt
	current_level_text = Text( **current_level_text_config )

	game_aborted = False	
	
	while not player1.is_dead and not player2.is_dead:
		# Load and run the next level.
		current_level = levels.LEVELS[current_level_cnt]

		# Create malware first!!!
		# Malware sprite group.
		malware = pygame.sprite.Group()
		
		# Create viruses for the current level.
		for virus in range( current_level["viruses"] * 2 ):
			malware.add( Malware( **Virus_Config.LEVELS[current_level_cnt] ) )
			
		# Create worms for the current level.
		for worm in range( current_level["worms"] * 2 ):
			malware.add( Malware( **Worm_Config.LEVELS[current_level_cnt] ) )

		# Create trojan horses for the current level.
		for worm in range( current_level["trojan_horses"] * 2 ):
			malware.add( Malware( **Trojan_Horse_Config.LEVELS[current_level_cnt] ) )

		# Create backdoors for the current level.
		for worm in range( current_level["backdoors"] * 2 ):
			malware.add( Malware( **Backdoor_Config.LEVELS[current_level_cnt] ) )

		# Create rootkits for the current level.
		for worm in range( current_level["rootkits"] * 2 ):
			malware.add( Malware( **Rootkit_Config.LEVELS[current_level_cnt] ) )	
		
		# Create passive data that move from the left to the right.
		passive_data = pygame.sprite.Group()
		for x in range( 0, 10 ):
			passive_data.add(
				Passive_Data_Sprite(
					"Neutral_Data" + str(random.randint(1,3)) + ".png",
					90,
					random.randint( 2, 6 ),
					3.0,
					pygame.Rect( -200, 59, 1324, 650 )
				)
			)

		# We inform the user about the next level.
		show_level_screen( window, clock, current_level_cnt )

		# The main gameloop runs the game and keeps it running until the user
		# chooses to quit the game.
		# We are going to use a variable to control this behaviour.		
		run_level = True
		while run_level:
			# We check for keypresses that come from the keyboard of the user.
			# If the user presses the ESC-key, we quit the game-loop, and thus, the
			# game.
			# A keypress is an event.  All events are stored in a list, which I call
			# the eventlist.
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
					if show_pause_screen( window, clock ):
						run_level = False
						player1.is_dead = True
						player2.is_dead = True
						game_aborted = True

			if player1.is_dead or player2.is_dead and not run_level:
				continue

			# We can draw on this window and fill it with colours.  The first thing
			# we need to do is fill the entire window with GREY1.  This is something
			# I like to do conveniently to keep the window refreshed.
			window.fill( config.GREY1 )		

			passive_data.update( items_group )
			items_group.update( players, allies_players, malware )			
			malware.update( allies_players )
			allies.update( malware )
			players.update( eventlist, malware )

			passive_data.draw( window )
			items_group.draw( window )			
					
			for m in malware:
				m.draw( window )
			for a in allies:
				a.draw( window )
			for p in players:
				p.draw( window )

			if len( malware ) == 0:
				run_level = False
			if len( players ) <= 1:
				run_level = False
				show_you_lose_screen( window, clock )				

			# Update the score displayed.
			current_score = helpers.sum_score( player1.get_score(), player2.get_score() )
			score_counter.change_text( config.SCORE_INDICATOR["_text"] % current_score )

			# Blit the score indicator / counter
			window.blit( score_counter.image, score_counter.rect )	
			window.blit( current_level_text.image, current_level_text.rect )				

			# Now that we have made changes to the window we need to show them to
			# the user who plays our game.
			pygame.display.update()

			for m in malware:
				if m.has_entered_cpu:
					run_level = False
					player1.is_dead = True
					player2.is_dead = True					
					show_you_lose_screen( window, clock )	
					break
					

			# We need to limit the game to a certain amount of updates per second.
			# If we would not do this, the computer would try to refresh or run
			# through the while-loop as quickly as possible.  This would result
			# in unstable playing, and also it would use so many CPU resources that
			# its' cooler would turn on, which can be really annoying.
			# We will run this game at a specified frame rate.
			clock.tick( config.FPS )


		if player1.is_dead or player2.is_dead and not game_aborted:
			# Check if the player still made it into the highscore.
			if helpers.is_in_highscore( current_score, "multi" ) and not game_aborted:
				# Show the screen where the player can enter his name, because
				# he made it into the highscore.
				player_name = show_enter_name_for_highscore_table_screen( window, clock, "multi" )
				helpers.insert_new_highscore( player_name, current_score, "multi" )			
			continue
		current_level_cnt += 1
		if current_level_cnt > max_levels:
			show_mission_accomplished_screen( window, clock, current_score )
			# Check if the player still made it into the highscore.
			if helpers.is_in_highscore( current_score, "multi" ) and not game_aborted:
				# Show the screen where the player can enter his name, because
				# he made it into the highscore.
				player_name = show_enter_name_for_highscore_table_screen( window, clock, "multi" )
				helpers.insert_new_highscore( player_name, current_score, "multi" )			
			break
		else:	
			# Reset the position of the allies.
			# And remove remaining projectiles.
			for a in allies:
				a.reset_position()
				a.projectiles.empty()

			# Always show the current level in the top-left corner below the score counter.
			current_level_text_config = copy.deepcopy( config.CURRENT_LEVEL_TEXT )
			current_level_text_config["_text"] %= current_level_cnt
			current_level_text = Text( **current_level_text_config )				
				

def show_level_screen( window, clock, next_level ):
	"""This shows the next level / wave.

	"""
	next_level_heading = copy.deepcopy( config.NEXT_LEVEL_HEADING )
	next_level_heading["_text"] = next_level_heading["_text"] % str( next_level )
	lvl_heading = Text( **next_level_heading )

	running = True
	while running:
		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
				running = False

		window.fill( config.GREY1 )
		window.blit( lvl_heading.image, lvl_heading.rect )

		pygame.display.update()
		clock.tick( config.FPS )


def show_mission_accomplished_screen( window, clock, _final_score ):
	"""This shows a text message when the player(s) have successfully mastered
	all missions / levels.

	"""
	mission_accomplished = Text( **config.MISSION_ACCOMPLISHED )
	mission_accomplished_final_score_config = copy.deepcopy( config.MISSION_ACCOMPLISHED_FINAL_SCORE )
	mission_accomplished_final_score_config["_text"] %= _final_score
	mission_accomplished_final_score = Text( **mission_accomplished_final_score_config )

	running = True
	while running:
		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
				running = False

		window.fill( config.GREY1 )
		window.blit( mission_accomplished.image, mission_accomplished.rect )
		window.blit( mission_accomplished_final_score.image, mission_accomplished_final_score.rect )

		pygame.display.update()
		clock.tick( config.FPS )


def show_you_lose_screen( window, clock ):
	"""This shows the "You lose" screen.

	"""
	mission_failed = Text( **config.MISSION_FAILED )

	running = True
	while running:
		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
				running = False

		window.fill( config.GREY1 )
		window.blit( mission_failed.image, mission_failed.rect )

		pygame.display.update()
		clock.tick( config.FPS )


def show_pause_screen( window, clock ):
	"""This shows the pause screen.  It includes a menu where the user can choose
	to either quit the current game, or continue.

	"""
	pause_menu = Menu( config.PAUSE_MENU )
	
	running = True
	while running:
		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
				running = False
				
		window.fill( config.GREY1 )

		pause_menu.update( eventlist )
		pause_menu.draw( window )

		if pause_menu.action == "abort_game":
			return True
		elif pause_menu.action == "continue_game":
			return False

		pygame.display.update()
		clock.tick( config.FPS )


def show_enter_name_for_highscore_table_screen( window, clock, single_multi ):
	"""This shows a screen where the user is asked to enter his name, because
	he made it into the highscore.

	"""
	if single_multi == "single":
		enter_name_for_highscore_text = Text( **config.ENTER_NAME_FOR_HIGHSCORE_TEXT_SINGLE )
	else:
		enter_name_for_highscore_text = Text( **config.ENTER_NAME_FOR_HIGHSCORE_TEXT_MULTI )

	user_input = ""	
	username_text = Text( **config.USERNAME_TEXT )
	
	running = True
	while running:
		eventlist = pygame.event.get()
		for event in eventlist:
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_RETURN:
					return user_input
				if event.key >= 97 and event.key <= 122:
					user_input += chr( event.key )
				elif event.key == 32: # Space
					user_input += " "
				elif event.key == 8 and len( user_input ) > 0: # backspace.
					user_input = user_input[:-1]
					
				username_text.change_text( user_input )
				
		window.fill( config.GREY1 )
		window.blit( enter_name_for_highscore_text.image, enter_name_for_highscore_text.rect )
		window.blit( username_text.image, username_text.rect )
		

		pygame.display.update()
		clock.tick( config.FPS )

			
main()
