#    Inside The Data Tunnel - My game for PyWeek 20.
#    Copyright (C) 2015  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Modules
# Standard Python modules.
import random
import copy

# Third party modules
import pygame

# Own modules
from classes.Moving_Sprite import Moving_Sprite
from config import config
from classes.Item import Item


# Initialise pygame, so that we can use its' modules.
pygame.init()


class Passive_Data_Sprite( Moving_Sprite ):
	"""A passive data sprite is a sprite that does move from the left to the
	right and occasionally, drops off gems and items, which the players and
	their allies can use.

	"""
	def __init__( self, _image, _direction, _speed, _turn_speed,
				  _allowed_area ):
		"""Initialising the object.  Setting some values and paramters.  The
		parameters that are passed on to the parent class are documented inside
		that very parent class.

		"""
		pos_center = (
			random.randint( 0, config.WINDOW_WIDTH ),
			random.randint( 59, config.WINDOW_HEIGHT - 59 )
		)
		super( Passive_Data_Sprite, self ).__init__( _image, pos_center,
													 _direction, _speed, 														 _turn_speed, _allowed_area )

		# We don't want to distract the player, this is why we draw the neutral
		# data sprites slightly transparent.
		self.image.set_alpha( 20 )
		self.original_image.set_alpha( 20 )
		self.rotate_image()

		
	def update( self, _item_group ):
		"""Updates the sprite (basically just moves the sprite from the left to
		the right.

		"""
		super( Passive_Data_Sprite, self ).update()
		
		if self.can_move():
			self.move()
			if self.has_reached_right_side():
				self.reset()
		else:
			self.calc_x_axis_speed()
			self.calc_y_axis_speed()


		# Occasionally, drop an item.
		if random.randint( 1, 4000 ) == 2000:
			item_to_create = random.choice( config.ITEM_CONFIG_OPTIONS )
			chosen_item = copy.deepcopy( config.ITEMS_CONFIG[item_to_create] )
			chosen_item["_position_center"] = self.rect.center 
			_item_group.add( Item( **chosen_item ) )


	def has_reached_right_side( self ):
		"""This function checks if the passive data sprite has disappeared into
		the right side of the window.  If it has, 'True' is returned, otherwise
		'False'.

		"""
		if self.rect.left > config.WINDOW_WIDTH:
			return True
		else:
			return False


	def reset( self ):
		"""This function resets the position of the passive data sprite, so that
		it can again come in on the left and go towards the right of the window.

		A new, random position will be chosen.
		
		"""
		self.set_position_center( ( random.randint( self.allowed_area.left, 0 - self.rect.width / 2 ),
									random.randint( self.allowed_area.top, self.allowed_area.bottom ) ) )
		
