#    Inside The Data Tunnel - My game for PyWeek 20.
#    Copyright (C) 2015  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Modules
# Standard Python modules
import random

# Third party modules
import pygame

# Own modules
from config import config
from classes.Active_Moving_Sprite import Active_Moving_Sprite
from classes.EP_Bar import EP_Bar


class Player2( Active_Moving_Sprite ):
	def __init__( self, _image, _pos_center, _direction, _speed, _turn_speed,
				  _allowed_area, _max_hitpoints, _current_hitpoints,
				  _projectile_speed, _projectile_image, _projectile_damage,
				  _controls, _ep_to_next_level, _auto_fire_interval ):
				  
		super( Player2, self ).__init__( _image, _pos_center, _direction, _speed,
										_turn_speed, _allowed_area,
										_max_hitpoints, _current_hitpoints,
				  						_projectile_speed, _projectile_image,
				  						_projectile_damage )

		self.controls = _controls

		self.ep_to_next_level = _ep_to_next_level
		self.current_ep = 0
		
		# Create the EP bar.
		self.ep_bar = EP_Bar(
			self.rect.width,
			5,
			( self.rect.left, self.rect.top - 13 ),
			config.PURPLE,
			config.DARK_PURPLE,
			self.ep_to_next_level
		)	

		# Levels.
		self.current_level = 0

		# Create the level indicator.
		self.create_level_indicator( self.current_level )

		# We count the score.
		self.score = 0

		# Auto fire variables.
		self.auto_fire_counter = 0
		self.auto_fire_time = 5 * config.FPS
		self.auto_fire_on = False
		self.auto_fire_interval = _auto_fire_interval
		self.auto_fire_interval_counter = 0		
				

	def update( self, _eventlist, _malware ):
		"""

		"""	
		for event in _eventlist:
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_w:
					self.controls[pygame.K_w] = True
				if event.key == pygame.K_s:
					self.controls[pygame.K_s] = True
				if event.key == pygame.K_a:
					self.controls[pygame.K_a] = True
				if event.key == pygame.K_d:
					self.controls[pygame.K_d] = True
				if event.key == pygame.K_LALT  and not self.auto_fire_on:
					self.controls[pygame.K_LALT] = True									

			if event.type == pygame.KEYUP:
				if event.key == pygame.K_w:
					self.controls[pygame.K_w] = False
				if event.key == pygame.K_s:
					self.controls[pygame.K_s] = False
				if event.key == pygame.K_a:
					self.controls[pygame.K_a] = False
				if event.key == pygame.K_d:
					self.controls[pygame.K_d] = False					


		if self.controls[pygame.K_w]:
			if self.rect.top > config.ACTIVE_PLAYING_AREA[1]:		
				self.rect.centery -= self.get_speed()

		if self.controls[pygame.K_s]:
			if self.rect.bottom < config.ACTIVE_PLAYING_AREA[3]:		
				self.rect.centery += self.get_speed()

		if self.controls[pygame.K_a]:
			if self.rect.left > config.ACTIVE_PLAYING_AREA[0]:		
				self.rect.centerx -= self.get_speed()

		if self.controls[pygame.K_d]:
			if self.rect.right < config.ACTIVE_PLAYING_AREA[2]:		
				self.rect.centerx += self.get_speed()

		if self.controls[pygame.K_LALT]  and not self.auto_fire_on:
			self.fire_projectile()
			self.controls[pygame.K_LALT] = False

		if len( self.projectiles ) > 0:
			self.projectiles.update()

		if self.is_dead:
			self.projectiles.empty()

		super( Player2, self ).update( _malware )

		# Update the EP bar.
		self.ep_bar.update(
			self.current_ep,
			( self.rect.left, self.rect.top - 13 )
		)

		# Check if we've reached the required amount of EP to proceed to the
		# next level.
		if self.ep_to_next_level <= self.current_ep:
		
			# We have reached the required amount.  Change to the next level.
			# Several properties (such as projectile strength) are now upgraded.
			# Also, tt takes more EP to reach the next level.
			self.ep_to_next_level += 10
			self.current_ep = 0
			self.ep_bar.renew( self.ep_to_next_level )

			self.upgrade_properties(
				self.max_hitpoints + 25,
				( self.projectile_damage[0] + 1, self.projectile_damage[1] + 1 )
			)

			# Change the level.
			self.current_level += 1

			# Change the text of the level indicator so that it shows the correct
			# level.
			new_text = config.LEVEL_INDICATOR["_text"] % str( self.current_level )
			self.level_indicator.change_text( new_text )

		self.level_indicator.reposition( ( self.rect.left, self.rect.top - 23 ) )		

		# Auto fire.
		if self.auto_fire_on:
			if self.auto_fire_counter > self.auto_fire_time:
				self.auto_fire_counter = 0
				self.auto_fire_on = False
				self.auto_fire_interval_counter = 0
			else:
				self.auto_fire_counter += 1

			if self.auto_fire_interval_counter >= self.auto_fire_interval:
				self.auto_fire_interval_counter = 0
				self.fire_projectile()
			else:
				self.auto_fire_interval_counter += 1


	def increase_score( self, _score_to_add ):
		"""Increases the score.  Called when the player kills an enemy.

		"""
		self.score += _score_to_add


	def get_score( self ):
		return self.score
		

	def execute_item_action( self, _item_type, _allies_and_players, _malware ):
		"""This function executes the action from an item.

		"""
		if _item_type == "auto_fire":
			self.auto_fire_on = True
		elif _item_type == "single_health":
			self.restore_hp( 50 )
		elif _item_type == "group_health":
			for x in _allies_and_players:
				x.restore_hp( 50 )
		elif _item_type == "health_distribution":
			# Count all HP of allies and players.
			all_hp = 0

			# Count how many players and allies there are.
			n_allies_players = len( _allies_and_players )
			
			for x in _allies_and_players:
				all_hp += x.current_hitpoints

			# Calculate how many HP each one gets.
			hp_for_one_entity = all_hp / n_allies_players

			# Apply the health points.
			for x in _allies_and_players:
				x.current_hitpoints = hp_for_one_entity

		elif _item_type == "can_not_move":
			# All malware can not move for 5 seconds.
			for m in _malware:
				if not m.can_not_move:
					m.can_not_move = True


	def draw( self, window ):
		super( Player2, self ).draw( window )
		window.blit( self.ep_bar.image, self.ep_bar.rect )
		window.blit( self.level_indicator.image, self.level_indicator.rect )		
