#    Inside The Data Tunnel - My game for PyWeek 20.
#    Copyright (C) 2015  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Modules
# Standard Python modules
import random

# Third party modules
import pygame

# Own modules
from config import config
from classes.Moving_Sprite import Moving_Sprite

pygame.init()


class Projectile( Moving_Sprite ):
	"""A projectile is an object fired by an active moving sprite whose purpose
	is to inflict damage, and eventually, destroy an active moving sprite.

	A projectile has a start position, a speed along the x and y planes aswell
	as a certain amount of damage which it inflicts upon an active moving sprite
	on collision.

	A projectile that has collided with either the walls of the data tunnel, an
	active moving sprite or has left the screen, is killed.

	"""
	def __init__( self, _image, _position_center, _speed, _damage, _direction,
				  _allowed_area ):
		"""Initialise some variables.

		Parameters
		
			_image
			pygame.Surface
			The image to use for the projectile.  This will be displayed on the
			window.

			_postion_center
			( int, int )
			Defines the start position of the projectile.

			_speed
			float
			Defines the speed of the projectile.

			_direction
			int
			The direction towards which the projectile should head to.  In
			degrees, will be converted to radians, because that's what Python
			uses for calculations.

			_allowed_area
			pygame.Rect
			A rectangle in which the projectile is allowed to exist and move.
			Once it leaves this area, it is desroyed automatically.
			
		"""
		#NOTE:  _turn_speed (0) is irrelevant in this case.
		super( Projectile, self ).__init__( _image, _position_center,
											_direction, _speed, 0,
				  							_allowed_area )

		self.damage = random.randint( _damage[0], _damage[1] )


	def update( self ):
		"""Updates the projectile and / or its' behaviour.

		"""
		# First we need to check if the projectile is still wtihin the allowed
		# area.  If it isn't, it is destroyed automatically.
		# Otherwise it is allowed to move on.
		self.update_collision_rectangle_position()
		if self.is_outside_allowed_area():
			self.kill()
		else:
			self.move()


	def draw( self, window ):
		window.blit( self.image, self.rect )
