#    Inside The Data Tunnel - My game for PyWeek 20.
#    Copyright (C) 2015  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Modules

# Third party Python modules
import pygame

# Import own modules
from modules import helpers

pygame.init()

allowed_area = pygame.Rect( 0, 59, 1024, 650 )


LEVELS = {
	5: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 100,
		"_current_hitpoints": 100,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [5, 10],
		"_shooting_interval": 4,
		"_kill_score": 100,
		"_additional_dodging_amount": 10,
		"_ep_on_projectile_hit": 10
	},
	6: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 120,
		"_current_hitpoints": 120,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [8, 13],
		"_shooting_interval": 4,
		"_kill_score": 120,
		"_additional_dodging_amount": 10,
		"_ep_on_projectile_hit": 12
	},
	7: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 140,
		"_current_hitpoints": 140,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [11, 16],
		"_shooting_interval": 4,
		"_kill_score": 140,
		"_additional_dodging_amount": 10,
		"_ep_on_projectile_hit": 14
	},
	8: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 160,
		"_current_hitpoints": 160,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [14, 19],
		"_shooting_interval": 4,
		"_kill_score": 160,
		"_additional_dodging_amount": 10,
		"_ep_on_projectile_hit": 16
	},
	9: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 180,
		"_current_hitpoints": 180,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [17, 20],
		"_shooting_interval": 4,
		"_kill_score": 180,
		"_additional_dodging_amount": 10,
		"_ep_on_projectile_hit": 18
	}	,
	10: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 200,
		"_current_hitpoints": 200,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [20, 23],
		"_shooting_interval": 4,
		"_kill_score": 200,
		"_additional_dodging_amount": 10,
		"_ep_on_projectile_hit": 20
	},
	13: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 220,
		"_current_hitpoints": 220,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [23, 26],
		"_shooting_interval": 4,
		"_kill_score": 220,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 22
	},
	14: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 240,
		"_current_hitpoints": 240,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [26, 29],
		"_shooting_interval": 4,
		"_kill_score": 240,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 24
	},
	15: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 260,
		"_current_hitpoints": 260,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [29, 32],
		"_shooting_interval": 4,
		"_kill_score": 260,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 26
	},
	16: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 280,
		"_current_hitpoints": 280,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [32, 35],
		"_shooting_interval": 4,
		"_kill_score": 280,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 28
	},
	17: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 300,
		"_current_hitpoints": 300,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [35, 38],
		"_shooting_interval": 4,
		"_kill_score": 300,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 30
	},
	18: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 320,
		"_current_hitpoints": 320,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [38, 41],
		"_shooting_interval": 4,
		"_kill_score": 320,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 31
	},
	19: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 340,
		"_current_hitpoints": 340,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [41, 44],
		"_shooting_interval": 4,
		"_kill_score": 340,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 33
	},
	20: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 360,
		"_current_hitpoints": 360,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [44, 47],
		"_shooting_interval": 4,
		"_kill_score": 360,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 35
	},
	21: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 380,
		"_current_hitpoints": 380,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [47, 50],
		"_shooting_interval": 4,
		"_kill_score": 380,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 37
	},
	22: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 400,
		"_current_hitpoints": 400,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [50, 53],
		"_shooting_interval": 4,
		"_kill_score": 400,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 39
	},
	23: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 420,
		"_current_hitpoints": 420,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [53, 56],
		"_shooting_interval": 4,
		"_kill_score": 420,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 41
	},
	24: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 440,
		"_current_hitpoints": 440,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [56, 59],
		"_shooting_interval": 4,
		"_kill_score": 440,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 42
	},
	25: {
		"_image": "Worm.png",
		"_direction": 90,
		"_speed": 2,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 460,
		"_current_hitpoints": 460,
		"_projectile_speed": 4,
		"_projectile_image": "Worm_Projectile.png",
		"_projectile_damage": [59, 62],
		"_shooting_interval": 4,
		"_kill_score": 460,
		"_additional_dodging_amount": 12,
		"_ep_on_projectile_hit": 44
	}
}
