#    Inside The Data Tunnel - My game for PyWeek 20.
#    Copyright (C) 2015  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Modules

# Third party Python modules
import pygame

# Import own modules
from modules import helpers

pygame.init()

allowed_area = pygame.Rect( 0, 59, 1024, 650 )


LEVELS = {
	16: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 200,
		"_current_hitpoints": 200,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [20, 40],
		"_shooting_interval": 3,
		"_kill_score": 250,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 32
	},
	17: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 250,
		"_current_hitpoints": 250,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [40, 60],
		"_shooting_interval": 3,
		"_kill_score": 250,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 52
	},
	18: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 300,
		"_current_hitpoints": 300,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [60, 80],
		"_shooting_interval": 3,
		"_kill_score": 300,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 72
	},
	19: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 350,
		"_current_hitpoints": 350,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [80, 100],
		"_shooting_interval": 3,
		"_kill_score": 350,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 92
	},
	20: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 400,
		"_current_hitpoints": 400,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [100, 120],
		"_shooting_interval": 3,
		"_kill_score": 400,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 112
	},
	21: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 450,
		"_current_hitpoints": 450,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [120, 140],
		"_shooting_interval": 3,
		"_kill_score": 450,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 132
	},
	22: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 500,
		"_current_hitpoints": 500,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [140, 160],
		"_shooting_interval": 3,
		"_kill_score": 500,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 152
	},
	23: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 550,
		"_current_hitpoints": 550,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [160, 180],
		"_shooting_interval": 3,
		"_kill_score": 550,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 172
	},
	24: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 600,
		"_current_hitpoints": 600,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [180, 200],
		"_shooting_interval": 3,
		"_kill_score": 200,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 192
	},
	25: {
		"_image": "Backdoor.png",
		"_direction": 90,
		"_speed": 3,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 650,
		"_current_hitpoints": 650,
		"_projectile_speed": 8,
		"_projectile_image": "Backdoor_Projectile.png",
		"_projectile_damage": [200, 220],
		"_shooting_interval": 3,
		"_kill_score": 650,
		"_additional_dodging_amount": 15,
		"_ep_on_projectile_hit": 212
	}
}
