#    Inside The Data Tunnel - My game for PyWeek 20.
#    Copyright (C) 2015  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Modules

# Third party Python modules
import pygame

# Import own modules
from modules import helpers

pygame.init()

allowed_area = pygame.Rect( 0, 59, 1024, 650 ) # For testing purposes only.


LEVELS = {
	1: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 10,
		"_current_hitpoints": 10,
		"_projectile_speed": 3,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [1, 2],
		"_shooting_interval": 7,
		"_kill_score": 10,
		"_additional_dodging_amount": 1,
		"_ep_on_projectile_hit": 1
	},
	2: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 20,
		"_current_hitpoints": 20,
		"_projectile_speed": 3,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [2, 3],
		"_shooting_interval": 7,
		"_kill_score": 20,
		"_additional_dodging_amount": 2,
		"_ep_on_projectile_hit": 2
	},
	3: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 30,
		"_current_hitpoints": 30,
		"_projectile_speed": 3,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [3, 4],
		"_shooting_interval": 7,
		"_kill_score": 30,
		"_additional_dodging_amount": 3,
		"_ep_on_projectile_hit": 3
	},
	4: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 40,
		"_current_hitpoints": 40,
		"_projectile_speed": 3,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [4, 5],
		"_shooting_interval": 7,
		"_kill_score": 40,
		"_additional_dodging_amount": 4,
		"_ep_on_projectile_hit": 4
	},
	5: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 50,
		"_current_hitpoints": 50,
		"_projectile_speed": 4,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [5, 6],
		"_shooting_interval": 6,
		"_kill_score": 50,
		"_additional_dodging_amount": 5,
		"_ep_on_projectile_hit": 5
	},
	6: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 60,
		"_current_hitpoints": 60,
		"_projectile_speed": 4,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [6, 7],
		"_shooting_interval": 6,
		"_kill_score": 60,
		"_additional_dodging_amount": 6,
		"_ep_on_projectile_hit": 6
	},
	7: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 70,
		"_current_hitpoints": 70,
		"_projectile_speed": 4,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [7, 8],
		"_shooting_interval": 6,
		"_kill_score": 70,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 7
	},
	8: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 80,
		"_current_hitpoints": 80,
		"_projectile_speed": 4,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [8, 9],
		"_shooting_interval": 6,
		"_kill_score": 80,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 8
	},
	9: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 90,
		"_current_hitpoints": 90,
		"_projectile_speed": 4,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [9, 10],
		"_shooting_interval": 6,
		"_kill_score": 90,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 9
	},
	10: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 100,
		"_current_hitpoints": 100,
		"_projectile_speed": 4,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [10, 11],
		"_shooting_interval": 6,
		"_kill_score": 100,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 10
	},
	11: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 110,
		"_current_hitpoints": 110,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [11, 12],
		"_shooting_interval": 5,
		"_kill_score": 110,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 11
	},
	12: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 120,
		"_current_hitpoints": 120,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [12, 13],
		"_shooting_interval": 5,
		"_kill_score": 120,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 12
	},
	13: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 130,
		"_current_hitpoints": 130,
		"_projectile_speed": 4,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [13, 14],
		"_shooting_interval": 6,
		"_kill_score": 130,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 13
	},
	14: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 140,
		"_current_hitpoints": 140,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [14, 15],
		"_shooting_interval": 5,
		"_kill_score": 140,
		"_additional_dodging_amount": 7,
		"_ep_on_projectile_hit": 14
	},
	15: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 150,
		"_current_hitpoints": 150,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [15, 16],
		"_shooting_interval": 5,
		"_kill_score": 150,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 15
	},
	16: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 160,
		"_current_hitpoints": 160,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [16, 17],
		"_shooting_interval": 5,
		"_kill_score": 160,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 16
	},
	17: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 170,
		"_current_hitpoints": 170,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [17, 18],
		"_shooting_interval": 5,
		"_kill_score": 170,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 17
	},
	18: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 180,
		"_current_hitpoints": 180,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [18, 19],
		"_shooting_interval": 5,
		"_kill_score": 180,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 18
	},
	19: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 190,
		"_current_hitpoints": 190,
		"_projectile_speed": 5,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [20, 21],
		"_shooting_interval": 5,
		"_kill_score": 190,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 19
	},
	20: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 200,
		"_current_hitpoints": 200,
		"_projectile_speed": 6,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [21, 23],
		"_shooting_interval": 5,
		"_kill_score": 200,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 20
	},
	21: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 210,
		"_current_hitpoints": 210,
		"_projectile_speed": 6,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [23, 25],
		"_shooting_interval": 4,
		"_kill_score": 210,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 21
	},
	22: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 220,
		"_current_hitpoints": 220,
		"_projectile_speed": 6,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [25, 27],
		"_shooting_interval": 4,
		"_kill_score": 220,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 22
	},
	23: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 230,
		"_current_hitpoints": 230,
		"_projectile_speed": 6,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [27, 29],
		"_shooting_interval": 4,
		"_kill_score": 230,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 23
	},
	24: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 240,
		"_current_hitpoints": 240,
		"_projectile_speed": 6,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [31, 33],
		"_shooting_interval": 4,
		"_kill_score": 240,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 24
	},
	25: {
		"_image": "Virus.png",
		"_direction": 90,
		"_speed": 1,
		"_turn_speed": 5,
		"_allowed_area": allowed_area,
		"_max_hitpoints": 250,
		"_current_hitpoints": 250,
		"_projectile_speed": 6,
		"_projectile_image": "Virus_Projectile.png",
		"_projectile_damage": [33, 35],
		"_shooting_interval": 4,
		"_kill_score": 250,
		"_additional_dodging_amount": 8,
		"_ep_on_projectile_hit": 25
	}
}
